@@ -17,6 +17,7 @@ module Dhall.Diff (
     , diff
     ) where
 
+import Data.ByteString       (ByteString)
 import Data.Foldable         (fold, toList)
 import Data.List.NonEmpty    (NonEmpty (..))
 import Data.Monoid           (Any (..))
@@ -40,16 +41,16 @@ import Dhall.Syntax
 import Numeric.Natural       (Natural)
 import Prettyprinter         (Doc, Pretty)
 
-import qualified Data.Algorithm.Diff   as Algo.Diff
+import qualified Data.Algorithm.Diff    as Algo.Diff
 import qualified Data.List.NonEmpty
 import qualified Data.Set
 import qualified Data.Text
-import qualified Data.Time             as Time
+import qualified Data.Time              as Time
 import qualified Dhall.Map
-import qualified Dhall.Normalize       as Normalize
-import qualified Dhall.Pretty.Internal as Internal
-import qualified Dhall.Syntax          as Syntax
-import qualified Prettyprinter         as Pretty
+import qualified Dhall.Normalize        as Normalize
+import qualified Dhall.Pretty.Internal  as Internal
+import qualified Dhall.Syntax           as Syntax
+import qualified Prettyprinter          as Pretty
 
 {-| This type is a `Doc` enriched with a `same` flag to efficiently track if
     any difference was detected
@@ -124,8 +125,8 @@ dot = token Internal.dot
 equals :: Diff
 equals = token Internal.equals
 
-forall :: Diff
-forall = token (Internal.forall Internal.Unicode)
+forall_ :: Diff
+forall_ = token (Internal.forall_ Internal.Unicode)
 
 lambda :: Diff
 lambda = token (Internal.lambda Internal.Unicode)
@@ -383,6 +384,10 @@ diffChunks cL cR
         (Right x, Right y) -> diff x y
         _                  -> diffTextSkeleton
 
+diffBytes :: ByteString -> ByteString -> Diff
+diffBytes l r =
+    "0x" <> diffText (Internal.prettyBase16 l) (Internal.prettyBase16 r)
+
 diffList
     :: (Eq a, Pretty a)
     => Seq (Expr Void a) -> Seq (Expr Void a) -> Diff
@@ -532,6 +537,10 @@ skeleton (BoolIf {}) =
     <>  keyword "else"
     <>  " "
     <>  ignore
+skeleton (BytesLit {}) =
+        "0x\""
+    <>  ignore
+    <>  "\""
 skeleton (NaturalPlus {}) =
         ignore
     <>  " "
@@ -737,7 +746,7 @@ diff l@(Pi {}) r@(Pi {}) =
             | same docA =
                 format mempty docB
             | otherwise =
-                    forall
+                    forall_
                 <>  lparen
                 <>  format " " docA
                 <>  colon
@@ -1169,6 +1178,18 @@ diffPrimitiveExpression l@Bool r =
     mismatch l r
 diffPrimitiveExpression l r@Bool =
     mismatch l r
+diffPrimitiveExpression Bytes Bytes =
+    "…"
+diffPrimitiveExpression l@Bytes r =
+    mismatch l r
+diffPrimitiveExpression l r@Bytes =
+    mismatch l r
+diffPrimitiveExpression (BytesLit l) (BytesLit r) =
+    diffBytes l r
+diffPrimitiveExpression l@(BytesLit {}) r =
+    mismatch l r
+diffPrimitiveExpression l r@(BytesLit {}) =
+    mismatch l r
 diffPrimitiveExpression Natural Natural =
     "…"
 diffPrimitiveExpression l@Natural r =
@@ -1289,18 +1310,36 @@ diffPrimitiveExpression l r@Date =
     mismatch l r
 diffPrimitiveExpression l@Date r=
     mismatch l r
+diffPrimitiveExpression DateShow DateShow =
+    "…"
+diffPrimitiveExpression l r@DateShow =
+    mismatch l r
+diffPrimitiveExpression l@DateShow r=
+    mismatch l r
 diffPrimitiveExpression Time Time =
     "…"
 diffPrimitiveExpression l r@Time =
     mismatch l r
 diffPrimitiveExpression l@Time r=
     mismatch l r
+diffPrimitiveExpression TimeShow TimeShow =
+    "…"
+diffPrimitiveExpression l r@TimeShow =
+    mismatch l r
+diffPrimitiveExpression l@TimeShow r=
+    mismatch l r
 diffPrimitiveExpression TimeZone TimeZone =
     "…"
 diffPrimitiveExpression l r@TimeZone =
     mismatch l r
 diffPrimitiveExpression l@TimeZone r=
     mismatch l r
+diffPrimitiveExpression TimeZoneShow TimeZoneShow =
+    "…"
+diffPrimitiveExpression l r@TimeZoneShow =
+    mismatch l r
+diffPrimitiveExpression l@TimeZoneShow r=
+    mismatch l r
 diffPrimitiveExpression List List =
     "…"
 diffPrimitiveExpression l@List r =
