@@ -46,13 +46,18 @@ module Dhall.Eval (
   , Val(..)
   , (~>)
   , textShow
+  , dateShow
+  , timeShow
+  , timezoneShow
   ) where
 
 import Data.Bifunctor     (first)
+import Data.ByteString    (ByteString)
 import Data.Foldable      (foldr', toList)
 import Data.List.NonEmpty (NonEmpty (..))
 import Data.Sequence      (Seq, ViewL (..), ViewR (..))
 import Data.Text          (Text)
+import Data.Time          (Day, TimeOfDay(..), TimeZone)
 import Data.Void          (Void)
 import Dhall.Map          (Map)
 import Dhall.Set          (Set)
@@ -80,7 +85,7 @@ import qualified Data.Time     as Time
 import qualified Dhall.Map     as Map
 import qualified Dhall.Set
 import qualified Dhall.Syntax  as Syntax
-import qualified Text.Printf
+import qualified Text.Printf   as Printf
 
 data Environment a
     = Empty
@@ -170,6 +175,9 @@ data Val a
     | VBoolNE !(Val a) !(Val a)
     | VBoolIf !(Val a) !(Val a) !(Val a)
 
+    | VBytes
+    | VBytesLit ByteString
+
     | VNatural
     | VNaturalLit !Natural
     | VNaturalFold !(Val a) !(Val a) !(Val a) !(Val a)
@@ -202,10 +210,13 @@ data Val a
 
     | VDate
     | VDateLiteral Time.Day
+    | VDateShow !(Val a)
     | VTime
     | VTimeLiteral Time.TimeOfDay Word
+    | VTimeShow !(Val a)
     | VTimeZone
     | VTimeZoneLiteral Time.TimeZone
+    | VTimeZoneShow !(Val a)
 
     | VList !(Val a)
     | VListLit !(Maybe (Val a)) !(Seq (Val a))
@@ -490,6 +501,10 @@ eval !env t0 =
                 (b', VBoolLit True, VBoolLit False) -> b'
                 (_, t', f') | conv env t' f'        -> t'
                 (b', t', f')                        -> VBoolIf b' t' f'
+        Bytes ->
+            VBytes
+        BytesLit b ->
+            VBytesLit b
         Natural ->
             VNatural
         NaturalLit n ->
@@ -659,14 +674,26 @@ eval !env t0 =
             VDate
         DateLiteral d ->
             VDateLiteral d
+        DateShow ->
+            VPrim $ \case
+                VDateLiteral d -> VTextLit (VChunks [] (dateShow d))
+                t              -> VDateShow t
         Time ->
             VTime
         TimeLiteral t p ->
             VTimeLiteral t p
+        TimeShow ->
+            VPrim $ \case
+                VTimeLiteral d p -> VTextLit (VChunks [] (timeShow d p))
+                t                -> VTimeShow t
         TimeZone ->
             VTimeZone
         TimeZoneLiteral z ->
             VTimeZoneLiteral z
+        TimeZoneShow ->
+            VPrim $ \case
+                VTimeZoneLiteral d -> VTextLit (VChunks [] (timezoneShow d))
+                t                  -> VTimeZoneShow t
         List ->
             VPrim VList
         ListLit ma ts ->
@@ -890,9 +917,32 @@ textShow text = "\"" <> Text.concatMap f text <> "\""
     f '\r' = "\\r"
     f '\t' = "\\t"
     f '\f' = "\\f"
-    f c | c <= '\x1F' = Text.pack (Text.Printf.printf "\\u%04x" (Data.Char.ord c))
+    f c | c <= '\x1F' = Text.pack (Printf.printf "\\u%04x" (Data.Char.ord c))
         | otherwise   = Text.singleton c
 
+-- | Utility that powers the @Date/show@ built-in
+dateShow :: Day -> Text
+dateShow = Text.pack . Time.formatTime Time.defaultTimeLocale "%0Y-%m-%d"
+
+-- | Utility that powers the @Time/show@ built-in
+timeShow :: TimeOfDay -> Word -> Text
+timeShow (TimeOfDay hh mm seconds) precision =
+    Text.pack (Printf.printf "%02d:%02d:%02d" hh mm ss <> suffix)
+  where
+    magnitude :: Integer
+    magnitude = 10 ^ precision
+
+    (ss, fraction) =
+        truncate (seconds * fromInteger magnitude) `divMod` magnitude
+
+    suffix
+        | precision == 0 = ""
+        | otherwise      = Printf.printf ".%0*d" precision fraction
+
+-- | Utility that powers the @TimeZone/show@ built-in
+timezoneShow :: TimeZone -> Text
+timezoneShow = Text.pack . Time.formatTime Time.defaultTimeLocale "%Ez"
+
 conv :: forall a. Eq a => Environment a -> Val a -> Val a -> Bool
 conv !env t0 t0' =
     case (t0, t0') of
@@ -940,6 +990,10 @@ conv !env t0 t0' =
             conv env t t' && conv env u u'
         (VBoolIf t u v, VBoolIf t' u' v') ->
             conv env t t' && conv env u u' && conv env v v'
+        (VBytes, VBytes) ->
+            True
+        (VBytesLit l, VBytesLit r) ->
+            l == r
         (VNatural, VNatural) ->
             True
         (VNaturalLit n, VNaturalLit n') ->
@@ -996,14 +1050,20 @@ conv !env t0 t0' =
             True
         (VDateLiteral l, VDateLiteral r) ->
             l == r
+        (VDateShow t, VDateShow t') ->
+            conv env t t'
         (VTime, VTime) ->
             True
         (VTimeLiteral tl pl, VTimeLiteral tr pr) ->
             tl == tr && pl == pr
+        (VTimeShow t, VTimeShow t') ->
+            conv env t t'
         (VTimeZone, VTimeZone) ->
             True
         (VTimeZoneLiteral l, VTimeZoneLiteral r) ->
             l == r
+        (VTimeZoneShow t, VTimeZoneShow t') ->
+            conv env t t'
         (VList a, VList a') ->
             conv env a a'
         (VListLit _ xs, VListLit _ xs') ->
@@ -1152,6 +1212,10 @@ quote !env !t0 =
             BoolNE (quote env t) (quote env u)
         VBoolIf t u v ->
             BoolIf (quote env t) (quote env u) (quote env v)
+        VBytes ->
+            Bytes
+        VBytesLit b ->
+            BytesLit b
         VNatural ->
             Natural
         VNaturalLit n ->
@@ -1208,14 +1272,20 @@ quote !env !t0 =
             Date
         VDateLiteral d ->
             DateLiteral d
+        VDateShow t ->
+            DateShow `qApp` t
         VTime ->
             Time
         VTimeLiteral t p ->
             TimeLiteral t p
+        VTimeShow t ->
+            TimeShow `qApp` t
         VTimeZone ->
             TimeZone
         VTimeZoneLiteral z ->
             TimeZoneLiteral z
+        VTimeZoneShow t ->
+            TimeZoneShow `qApp` t
         VList t ->
             List `qApp` t
         VListLit ma ts ->
@@ -1351,6 +1421,10 @@ alphaNormalize = goEnv EmptyNames
                 BoolNE  (go t) (go u)
             BoolIf b t f ->
                 BoolIf  (go b) (go t) (go f)
+            Bytes ->
+                Bytes
+            BytesLit b ->
+                BytesLit b
             Natural ->
                 Natural
             NaturalLit n ->
@@ -1407,14 +1481,20 @@ alphaNormalize = goEnv EmptyNames
                 Date
             DateLiteral d ->
                 DateLiteral d
+            DateShow ->
+                DateShow
             Time ->
                 Time
             TimeLiteral t p ->
                 TimeLiteral t p
+            TimeShow ->
+                TimeShow
             TimeZone ->
                 TimeZone
             TimeZoneLiteral z ->
                 TimeZoneLiteral z
+            TimeZoneShow ->
+                TimeZoneShow
             List ->
                 List
             ListLit ma ts ->
