@@ -36,6 +36,7 @@ import Dhall.Import
     , SemanticCacheMode (..)
     , _semanticCacheMode
     )
+import Dhall.Package       (writePackage)
 import Dhall.Parser        (Src)
 import Dhall.Pretty
     ( Ann
@@ -159,9 +160,10 @@ data Mode
     | Encode { file :: Input, json :: Bool }
     | Decode { file :: Input, json :: Bool, quiet :: Bool }
     | Text { file :: Input, output :: Output }
-    | DirectoryTree { file :: Input, path :: FilePath }
+    | DirectoryTree { allowSeparators :: Bool, file :: Input, path :: FilePath }
     | Schemas { file :: Input, outputMode :: OutputMode, schemas :: Text }
     | SyntaxTree { file :: Input, noted :: Bool }
+    | Package { name :: Maybe String, files :: NonEmpty FilePath }
 
 -- | This specifies how to resolve transitive dependencies
 data ResolveMode
@@ -269,7 +271,7 @@ parseMode =
             Generate
             "to-directory-tree"
             "Convert nested records of Text literals into a directory tree"
-            (DirectoryTree <$> parseFile <*> parseDirectoryTreeOutput)
+            (DirectoryTree <$> parseDirectoryTreeAllowSeparators <*> parseFile <*> parseDirectoryTreeOutput)
     <|> subcommand
             Interpret
             "resolve"
@@ -310,6 +312,11 @@ parseMode =
             "hash"
             "Compute semantic hashes for Dhall expressions"
             (Hash <$> parseFile <*> parseCache)
+    <|> subcommand
+            Miscellaneous
+            "package"
+            "Create a package.dhall referencing the provided paths"
+            (Package <$> parsePackageName <*> parsePackageFiles)
     <|> subcommand
             Miscellaneous
             "tags"
@@ -506,7 +513,7 @@ parseMode =
     parseAllFlag =
         Options.Applicative.switch
         (   Options.Applicative.long "all"
-        <>  Options.Applicative.help "Add integrity checks to all imports (not just remote imports)"
+        <>  Options.Applicative.help "Add integrity checks to all imports (not just remote imports) except for missing imports"
         )
 
     parseCacheFlag =
@@ -533,6 +540,12 @@ parseMode =
             <>  Options.Applicative.metavar "EXPR"
             )
 
+    parseDirectoryTreeAllowSeparators =
+        Options.Applicative.switch
+            (   Options.Applicative.long "allow-path-separators"
+            <>  Options.Applicative.help "Whether to allow path separators in file names"
+            )
+
     parseDirectoryTreeOutput =
         Options.Applicative.strOption
             (   Options.Applicative.long "output"
@@ -553,6 +566,22 @@ parseMode =
             <>  Options.Applicative.help "Cache the hashed expression"
             )
 
+    parsePackageName = optional $
+        Options.Applicative.strOption
+            (   Options.Applicative.long "name"
+            <>  Options.Applicative.help "The filename of the package"
+            <>  Options.Applicative.metavar "NAME"
+            <>  Options.Applicative.action "file"
+            )
+
+    parsePackageFiles = (:|) <$> p <*> Options.Applicative.many p
+      where
+        p = Options.Applicative.strArgument
+                (   Options.Applicative.help "Paths that may either point to files or directories. If the latter is the case all *.dhall files in the directory will be included."
+                <>  Options.Applicative.metavar "PATH"
+                <>  Options.Applicative.action "file"
+                )
+
 -- | `ParserInfo` for the `Options` type
 parserInfoOptions :: ParserInfo Options
 parserInfoOptions =
@@ -997,7 +1026,7 @@ command (Options {..}) = do
 
             let normalizedExpression = Dhall.Core.normalize resolvedExpression
 
-            DirectoryTree.toDirectoryTree path normalizedExpression
+            DirectoryTree.toDirectoryTree allowSeparators path normalizedExpression
 
         Dhall.Main.Schemas{..} ->
             Dhall.Schemas.schemasCommand Dhall.Schemas.Schemas{ input = file, ..}
@@ -1012,6 +1041,8 @@ command (Options {..}) = do
                     denoted = Dhall.Core.denote expression
                 in Text.Pretty.Simple.pPrintNoColor denoted
 
+        Package {..} -> writePackage (fromMaybe Unicode chosenCharacterSet) name files
+
 -- | Entry point for the @dhall@ executable
 main :: IO ()
 main = do
