@@ -1,10 +1,8 @@
-{-# LANGUAGE CPP                #-}
 {-# LANGUAGE DeriveAnyClass     #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE DeriveGeneric      #-}
 {-# LANGUAGE DeriveLift         #-}
-{-# LANGUAGE DeriveTraversable  #-}
-{-# LANGUAGE RecordWildCards    #-}
+{-# LANGUAGE ExplicitForAll     #-}
 {-# LANGUAGE TypeFamilies       #-}
 
 -- | `Map` type used to represent records and unions
@@ -18,6 +16,7 @@ module Dhall.Map
     , singleton
     , fromList
     , fromListWithKey
+    , fromMap
 
       -- * Constructing unordered 'Map's
     , unorderedSingleton
@@ -34,6 +33,7 @@ module Dhall.Map
       -- * Deletion/Update
     , delete
     , filter
+    , partition
     , restrictKeys
     , withoutKeys
     , mapMaybe
@@ -68,14 +68,13 @@ module Dhall.Map
     , elems
     ) where
 
-import Control.Applicative ((<|>))
-import Control.DeepSeq (NFData)
-import Data.Data (Data)
-import Data.Semigroup
-import GHC.Generics (Generic)
-import Instances.TH.Lift ()
+import Control.Applicative        ((<|>))
+import Control.DeepSeq            (NFData)
+import Data.Data                  (Data)
+import GHC.Generics               (Generic)
+import Instances.TH.Lift          ()
 import Language.Haskell.TH.Syntax (Lift)
-import Prelude hiding (filter, lookup)
+import Prelude                    hiding (filter, lookup)
 
 import qualified Data.List
 import qualified Data.Map
@@ -134,7 +133,7 @@ instance Ord k => Traversable (Map k) where
 {-|
 prop> \x y z -> x <> (y <> z) == (x <> y) <> (z :: Map Int Int)
 -}
-instance Ord k => Data.Semigroup.Semigroup (Map k v) where
+instance Ord k => Semigroup (Map k v) where
     (<>) = union
     {-# INLINABLE (<>) #-}
 
@@ -146,11 +145,6 @@ instance Ord k => Monoid (Map k v) where
     mempty = Map Data.Map.empty (Original [])
     {-# INLINABLE mempty #-}
 
-#if !(MIN_VERSION_base(4,11,0))
-    mappend = (<>)
-    {-# INLINABLE mappend #-}
-#endif
-
 instance (Show k, Show v, Ord k) => Show (Map k v) where
     showsPrec d m =
         showParen (d > 10) (showString "fromList " . showsPrec 11 kvs)
@@ -218,6 +212,10 @@ fromListWithKey f kvs = Map m ks
     ks = Original (nubOrd (map fst kvs))
 {-# INLINABLE fromListWithKey #-}
 
+-- | Create a `Map` from a @"Data.Map".`Data.Map.Map`@
+fromMap :: Data.Map.Map k v -> Map k v
+fromMap m = Map m Sorted
+
 {-| Remove duplicates from a  list
 
 >>> nubOrd [1,2,3]
@@ -359,7 +357,22 @@ filter predicate (Map m ks) = Map m' ks'
     ks' = filterKeys (\k -> Data.Map.member k m') ks
 {-# INLINABLE filter #-}
 
-{-| Restrict a 'Map' to only those keys found in a @"Data.Set".'Set'@.
+{-| Split the map into values that do and don't satisfy the predicate
+
+>>> partition even (fromList [("C",3),("B",2),("A",1)])
+(fromList [("B",2)],fromList [("C",3),("A",1)])
+>>> partition odd (fromList [("C",3),("B",2),("A",1)])
+(fromList [("C",3),("A",1)],fromList [("B",2)])
+-}
+partition :: Ord k => (a -> Bool) -> Map k a -> (Map k a, Map k a)
+partition predicate (Map m ks) = (Map mpass kpass, Map mfail kfail)
+  where
+    (mpass, mfail) = Data.Map.partition predicate m
+
+    (kpass, kfail) = partitionKeys (\k -> Data.Map.member k mpass) ks
+{-# INLINABLE partition #-}
+
+{-| Restrict a 'Map' to only those keys found in a @"Data.Set".'Data.Set.Set'@.
 
 >>> restrictKeys (fromList [("A",1),("B",2)]) (Data.Set.fromList ["A"])
 fromList [("A",1)]
@@ -367,16 +380,12 @@ fromList [("A",1)]
 restrictKeys :: Ord k => Map k a -> Data.Set.Set k -> Map k a
 restrictKeys (Map m ks) s = Map m' ks'
   where
-#if MIN_VERSION_containers(0,5,8)
     m' = Data.Map.restrictKeys m s
-#else
-    m' = Data.Map.filterWithKey (\k _ -> Data.Set.member k s) m
-#endif
 
     ks' = filterKeys (\k -> Data.Set.member k s) ks
 {-# INLINABLE restrictKeys #-}
 
-{-| Remove all keys in a @"Data.Set".'Set'@ from a 'Map'
+{-| Remove all keys in a @"Data.Set".'Data.Set.Set'@ from a 'Map'
 
 >>> withoutKeys (fromList [("A",1),("B",2)]) (Data.Set.fromList ["A"])
 fromList [("B",2)]
@@ -384,11 +393,7 @@ fromList [("B",2)]
 withoutKeys :: Ord k => Map k a -> Data.Set.Set k -> Map k a
 withoutKeys (Map m ks) s = Map m' ks'
   where
-#if MIN_VERSION_containers(0,5,8)
     m' = Data.Map.withoutKeys m s
-#else
-    m' = Data.Map.filterWithKey (\k _ -> Data.Set.notMember k s) m
-#endif
 
     ks' = filterKeys (\k -> Data.Set.notMember k s) ks
 {-# INLINABLE withoutKeys #-}
@@ -690,7 +695,7 @@ elems (Map m Sorted)        = Data.Map.elems m
 elems (Map m (Original ks)) = fmap (\k -> m Data.Map.! k) ks
 {-# INLINABLE elems #-}
 
-{-| Return the @"Data.Set".'Set'@ of the keys
+{-| Return the @"Data.Set".'Data.Set.Set'@ of the keys
 
 >>> keysSet (fromList [("B",1),("A",2)])
 fromList ["A","B"]
@@ -704,6 +709,13 @@ filterKeys _ Sorted        = Sorted
 filterKeys f (Original ks) = Original (Prelude.filter f ks)
 {-# INLINABLE filterKeys #-}
 
+partitionKeys :: (a -> Bool) -> Keys a -> (Keys a, Keys a)
+partitionKeys _ Sorted        = (Sorted, Sorted)
+partitionKeys f (Original ks) =
+    let (kpass, kfail) = Data.List.partition f ks
+    in (Original kpass, Original kfail)
+{-# INLINABLE partitionKeys #-}
+
 {- $setup
 >>> import Test.QuickCheck (Arbitrary(..), oneof)
 >>> :{
