@@ -51,6 +51,10 @@ module Dhall.Marshal.Decode
     , int64
     , scientific
     , double
+      -- ** Bytes
+    , lazyBytes
+    , strictBytes
+    , shortBytes
       -- ** Textual
     , string
     , lazyText
@@ -146,6 +150,7 @@ import Data.Functor.Contravariant
     , Op (..)
     , Predicate (..)
     )
+import Data.Functor.Identity            (Identity (..))
 import Data.Hashable                    (Hashable)
 import Data.List.NonEmpty               (NonEmpty (..))
 import Data.Typeable                    (Proxy (..), Typeable)
@@ -163,6 +168,9 @@ import Prelude                          hiding (maybe, sequence)
 import Prettyprinter                    (Pretty)
 
 import qualified Control.Applicative
+import qualified Data.ByteString
+import qualified Data.ByteString.Lazy
+import qualified Data.ByteString.Short
 import qualified Data.Foldable
 import qualified Data.Functor.Compose
 import qualified Data.Functor.Product
@@ -302,6 +310,15 @@ instance FromDhall Scientific where
 instance FromDhall Double where
     autoWith _ = double
 
+instance FromDhall Data.ByteString.Short.ShortByteString where
+    autoWith _ = shortBytes
+
+instance FromDhall Data.ByteString.Lazy.ByteString where
+    autoWith _ = lazyBytes
+
+instance FromDhall Data.ByteString.ByteString where
+    autoWith _ = strictBytes
+
 instance {-# OVERLAPS #-} FromDhall [Char] where
     autoWith _ = string
 
@@ -314,6 +331,9 @@ instance FromDhall Data.Text.Lazy.Text where
 instance FromDhall Text where
     autoWith _ = strictText
 
+instance FromDhall a => FromDhall (Identity a) where
+    autoWith opts = Identity <$> autoWith opts
+
 instance FromDhall a => FromDhall (Maybe a) where
     autoWith opts = maybe (autoWith opts)
 
@@ -917,6 +937,35 @@ double = Decoder {..}
 
     expected = pure Double
 
+{-| Decode a `Data.ByteString.Short.ShortByteString`
+
+>>> input shortBytes "0x\"00FF\""
+"\NUL\255"
+-}
+shortBytes :: Decoder Data.ByteString.Short.ShortByteString
+shortBytes = fmap Data.ByteString.Short.toShort strictBytes
+
+{-| Decode a lazy `Data.ByteString.Lazy.ByteString`.
+
+>>> input lazyBytes "0x\"00FF\""
+"\NUL\255"
+-}
+lazyBytes :: Decoder Data.ByteString.Lazy.ByteString
+lazyBytes = fmap Data.ByteString.Lazy.fromStrict strictBytes
+
+{-| Decode a strict `Data.ByteString.ByteString`
+
+>>> input strictBytes "0x\"00FF\""
+"\NUL\255"
+-}
+strictBytes :: Decoder Data.ByteString.ByteString
+strictBytes = Decoder {..}
+  where
+    extract (BytesLit b) = pure b
+    extract  expr        = typeError expected expr
+
+    expected = pure Bytes
+
 {-| Decode `Data.Text.Short.ShortText`.
 
 >>> input shortText "\"Test\""
@@ -925,7 +974,7 @@ double = Decoder {..}
 shortText :: Decoder Data.Text.Short.ShortText
 shortText = fmap Data.Text.Short.fromText strictText
 
-{-| Decode lazy `Data.Text.Text`.
+{-| Decode lazy `Data.Text.Lazy.Text`.
 
 >>> input lazyText "\"Test\""
 "Test"
@@ -1550,6 +1599,7 @@ data ExtractError s a =
     TypeMismatch (InvalidDecoder s a)
   | ExpectedTypeError ExpectedTypeError
   | ExtractError Text
+  deriving (Eq)
 
 instance (Pretty s, Pretty a, Typeable s, Typeable a) => Show (ExtractError s a) where
   show (TypeMismatch e)      = show e
@@ -1614,7 +1664,7 @@ data InvalidDecoder s a = InvalidDecoder
   { invalidDecoderExpected   :: Expr s a
   , invalidDecoderExpression :: Expr s a
   }
-  deriving (Typeable)
+  deriving (Eq, Typeable)
 
 instance (Pretty s, Typeable s, Pretty a, Typeable a) => Exception (InvalidDecoder s a)
 
