@@ -41,6 +41,7 @@ module Dhall.Pretty.Internal (
     , prettyDouble
     , prettyToStrictText
     , prettyToString
+    , prettyBase16
     , layout
     , layoutOpts
 
@@ -55,7 +56,7 @@ module Dhall.Pretty.Internal (
     , comma
     , dot
     , equals
-    , forall
+    , forall_
     , label
     , lambda
     , langle
@@ -72,30 +73,39 @@ module Dhall.Pretty.Internal (
     , temporalToText
     ) where
 
-import Control.DeepSeq            (NFData)
-import Data.Aeson                 (FromJSON (..), Value (String))
-import Data.Aeson.Types           (typeMismatch, unexpected)
-import Data.Data                  (Data)
-import Data.Foldable
-import Data.List.NonEmpty         (NonEmpty (..))
-import Data.Text                  (Text)
-import Dhall.Map                  (Map)
-import Dhall.Optics               (cosmosOf, foldOf, to)
-import Dhall.Src                  (Src (..))
-import Dhall.Syntax
-import GHC.Generics               (Generic)
-import Language.Haskell.TH.Syntax (Lift)
-import Numeric.Natural            (Natural)
-import Prettyprinter              (Doc, Pretty, space)
+import                Control.DeepSeq               (NFData)
+import                Data.Aeson
+    ( FromJSON (..)
+    , Value (String)
+    )
+import                Data.Aeson.Types              (typeMismatch, unexpected)
+import                Data.ByteString               (ByteString)
+import                Data.Data                     (Data)
+import                Data.Foldable
+import                Data.List.NonEmpty            (NonEmpty (..))
+import                Data.Text                     (Text)
+import                Dhall.Map                     (Map)
+import                Dhall.Optics                  (cosmosOf, foldOf, to)
+import                Dhall.Src                     (Src (..))
+import                Dhall.Syntax
+import {-# SOURCE #-} Dhall.Syntax.Instances.Pretty ()
+import                GHC.Generics                  (Generic)
+import                Language.Haskell.TH.Syntax    (Lift)
+import                Numeric.Natural               (Natural)
+import                Prettyprinter                 (Doc, Pretty, space)
 
+import qualified Control.Exception             as Exception
+import qualified Data.ByteString.Base16        as Base16
 import qualified Data.Char
 import qualified Data.HashSet
 import qualified Data.List                     as List
 import qualified Data.List.NonEmpty            as NonEmpty
 import qualified Data.Maybe
 import qualified Data.Text                     as Text
+import qualified Data.Text.Encoding            as Encoding
 import qualified Data.Time                     as Time
 import qualified Dhall.Map                     as Map
+import qualified Dhall.Syntax.Operations       as Operations
 import qualified Prettyprinter                 as Pretty
 import qualified Prettyprinter.Render.String   as Pretty
 import qualified Prettyprinter.Render.Terminal as Terminal
@@ -306,9 +316,9 @@ lambda :: CharacterSet -> Doc Ann
 lambda Unicode = syntax "λ"
 lambda ASCII   = syntax "\\"
 
-forall :: CharacterSet -> Doc Ann
-forall Unicode = syntax "∀"
-forall ASCII   = syntax "forall "
+forall_ :: CharacterSet -> Doc Ann
+forall_ Unicode = syntax "∀"
+forall_ ASCII   = syntax "forall "
 
 rarrow :: CharacterSet -> Doc Ann
 rarrow Unicode = syntax "→"
@@ -465,7 +475,7 @@ enclose beginShort beginLong sepShort sepLong endShort endLong docs =
     combineShort x y = x <> y
 
 {-| Format an expression that holds a variable number of elements without a
-    trailing document such as nested `let`, nested lambdas, or nested `forall`s
+    trailing document such as nested @let@, nested lambdas, or nested @forall@s
 -}
 enclose'
     :: Doc ann
@@ -794,7 +804,7 @@ prettyPrinters characterSet =
         docs (Pi _ "_" b c) = prettyOperatorExpression b : docs c
         docs (Pi _ a   b c) = Pretty.group (Pretty.flatAlt long short) : docs c
           where
-            long =  forall characterSet <> space
+            long =  forall_ characterSet <> space
                 <>  Pretty.align
                     (   lparen <> space
                     <>  prettyLabel a
@@ -805,7 +815,7 @@ prettyPrinters characterSet =
                     <>  rparen
                     )
 
-            short = forall characterSet <> lparen
+            short = forall_ characterSet <> lparen
                 <>  prettyLabel a
                 <>  space <> colon <> space
                 <>  prettyExpression b
@@ -1281,6 +1291,8 @@ prettyPrinters characterSet =
         prettyConst k
     prettyPrimitiveExpression Bool =
         builtin "Bool"
+    prettyPrimitiveExpression Bytes =
+        builtin "Bytes"
     prettyPrimitiveExpression Natural =
         builtin "Natural"
     prettyPrimitiveExpression NaturalFold =
@@ -1331,6 +1343,8 @@ prettyPrinters characterSet =
             )
       where
         (_HHHH, _MM, _DD) = Time.toGregorian day
+    prettyPrimitiveExpression DateShow =
+        builtin "Date/show"
     prettyPrimitiveExpression Time =
         builtin "Time"
     prettyPrimitiveExpression (TimeLiteral (Time.TimeOfDay hh mm seconds) precision) =
@@ -1350,7 +1364,9 @@ prettyPrinters characterSet =
 
         suffix
             | precision == 0 = ""
-            | otherwise      = "." <> Pretty.pretty fraction
+            | otherwise      = "." <> Pretty.pretty (Printf.printf "%0*d" precision fraction :: String)
+    prettyPrimitiveExpression TimeShow =
+        builtin "Time/show"
     prettyPrimitiveExpression TimeZone =
         builtin "TimeZone"
     prettyPrimitiveExpression (TimeZoneLiteral (Time.TimeZone minutes _ _)) =
@@ -1364,6 +1380,8 @@ prettyPrinters characterSet =
         sign = if 0 <= minutes then "+" else "-"
 
         (_HH, _MM) = abs minutes `divMod` 60
+    prettyPrimitiveExpression TimeZoneShow =
+        builtin "TimeZone/show"
     prettyPrimitiveExpression List =
         builtin "List"
     prettyPrimitiveExpression ListBuild =
@@ -1397,6 +1415,8 @@ prettyPrinters characterSet =
         prettyDouble a
     prettyPrimitiveExpression (TextLit a) =
         prettyChunks a
+    prettyPrimitiveExpression (BytesLit a) =
+        prettyBytes a
     prettyPrimitiveExpression (Record a) =
         prettyRecord a
     prettyPrimitiveExpression (RecordLit a) =
@@ -1608,6 +1628,10 @@ prettyPrinters characterSet =
     prettyUnion =
         angles . map prettyAlternative . Map.toList
 
+    prettyBytes :: ByteString -> Doc Ann
+    prettyBytes bytes =
+        literal (Pretty.pretty ("0x\"" <> prettyBase16 bytes <> "\""))
+
     prettyChunks :: Pretty a => Chunks Src a -> Doc Ann
     prettyChunks chunks@(Chunks a b)
         | anyText (== '\n') =
@@ -1928,6 +1952,15 @@ temporalToText e = case e of
 
     rendered = Just (prettyToStrictText e)
 
+prettyBase16 :: ByteString -> Text
+prettyBase16 bytes =
+    case Encoding.decodeUtf8' (Base16.encode bytes) of
+        Left exception ->
+            Operations.internalError
+                ("prettyBase16: base16-encoded bytes could not be decoded as UTF-8 text: " <> Text.pack (Exception.displayException exception))
+        Right text ->
+            Text.toUpper text
+
 {- $setup
 >>> import Test.QuickCheck (Fun(..))
 -}
