@@ -1,36 +1,38 @@
+{-# LANGUAGE BangPatterns        #-}
 {-# LANGUAGE DeriveAnyClass      #-}
-{-# LANGUAGE DeriveFoldable      #-}
 {-# LANGUAGE DeriveFunctor       #-}
 {-# LANGUAGE DeriveGeneric       #-}
-{-# LANGUAGE DeriveTraversable   #-}
 {-# LANGUAGE FlexibleContexts    #-}
 {-# LANGUAGE OverloadedLists     #-}
 {-# LANGUAGE OverloadedStrings   #-}
 {-# LANGUAGE RecordWildCards     #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE StandaloneDeriving  #-}
-{-# LANGUAGE TemplateHaskell     #-}
 {-# LANGUAGE TypeFamilies        #-}
 
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
 module Dhall.Test.Dhall where
 
-import Control.Exception (SomeException, try)
-import Data.Fix (Fix(..))
-import Data.List.NonEmpty (NonEmpty (..))
-import Data.Sequence (Seq)
-import Data.Scientific (Scientific)
-import Data.Text (Text)
-import Data.Vector (Vector)
-import Data.Void (Void)
-import Dhall (ToDhall, FromDhall)
-import Dhall.Core (Expr(..))
-import GHC.Generics (Generic, Rep)
-import Numeric.Natural (Natural)
+import Control.Exception      (SomeException, throwIO, try)
+import Data.Either.Validation (validationToEither)
+import Data.Fix               (Fix (..))
+import Data.Functor.Classes   (Eq1 (..), Show1 (..))
+import Data.List.NonEmpty     (NonEmpty (..))
+import Data.Maybe             (isJust)
+import Data.Scientific        (Scientific)
+import Data.Sequence          (Seq)
+import Data.Text              (Text)
+import Data.Vector            (Vector)
+import Data.Void              (Void)
+import Dhall                  (FromDhall, ToDhall)
+import Dhall.Core             (Expr (..))
+import GHC.Generics           (Generic, Rep)
+import Numeric.Natural        (Natural)
+import System.Timeout         (timeout)
 import Test.Tasty
 import Test.Tasty.HUnit
 
+import qualified Data.Functor.Classes as Classes
 import qualified Data.Text.Lazy
 import qualified Dhall
 import qualified Dhall.Core
@@ -42,7 +44,18 @@ data ExprF expr
    = LitF Natural
    | AddF expr expr
    | MulF expr expr
-   deriving (Eq, Functor, Generic, FromDhall, Show)
+   deriving (Eq, Functor, Generic, FromDhall, ToDhall, Show)
+
+instance Eq1 ExprF where
+    liftEq _  (LitF aL) (LitF aR) = aL == aR
+    liftEq eq (AddF aL bL) (AddF aR bR) = (aL `eq` aR) && (bL `eq` bR)
+    liftEq eq (MulF aL bL) (MulF aR bR) = (aL `eq` aR) && (bL `eq` bR)
+    liftEq _ _ _ = False
+
+instance Show1 ExprF where
+    liftShowsPrec _  _ d (LitF a) = showsPrec d a
+    liftShowsPrec sp _ d (AddF a b) = Classes.showsBinaryWith sp sp "AddF" d a b
+    liftShowsPrec sp _ d (MulF a b) = Classes.showsBinaryWith sp sp "MulF" d a b
 
 tests :: TestTree
 tests =
@@ -61,12 +74,12 @@ tests =
 data MyType = MyType { foo :: String , bar :: Natural }
 
 wrongDhallType :: Dhall.Decoder MyType
-wrongDhallType = Dhall.Decoder { .. }
-  where expected =
+wrongDhallType = Dhall.Decoder {..}
+  where expected = pure $
           Dhall.Core.Record
             ( Dhall.Map.fromList
-              [ ( "bar", Dhall.Core.Natural)
-              , ( "foo", Dhall.Core.Text )
+              [ ( "bar", Dhall.Core.makeRecordField Dhall.Core.Natural)
+              , ( "foo", Dhall.Core.makeRecordField Dhall.Core.Text )
               ]
             )
         extract expr = Dhall.typeError expected expr
@@ -79,14 +92,15 @@ shouldShowDetailedTypeError = testCase "detailed TypeError" $ do
   let expectedMsg =
         "\ESC[1;31mError\ESC[0m: Invalid Dhall.Decoder                                               \n\
         \                                                                                \n\
-        \Every Decoder must provide an extract function that succeeds if an expression   \n\
-        \matches the expected type.  You provided a Decoder that disobeys this contract  \n\
+        \Every Decoder must provide an extract function that does not fail with a type   \n\
+        \error if an expression matches the expected type.  You provided a Decoder that  \n\
+        \disobeys this contract                                                          \n\
         \                                                                                \n\
         \The Decoder provided has the expected dhall type:                               \n\
         \                                                                                \n\
         \↳ { bar : Natural, foo : Text }\n\
         \                                                                                \n\
-        \and it couldn't extract a value from the well-typed expression:                 \n\
+        \and it threw a type error during extraction from the well-typed expression:     \n\
         \                                                                                \n\
         \↳ { bar = 0, foo = \"foo\" }\n\
         \                                                                                \n"
@@ -117,18 +131,39 @@ shouldTreatAConstructorStoringUnitAsEmptyAlternative = testCase "Handle unit con
     let exampleEncoder :: Dhall.Encoder ()
         exampleEncoder = Dhall.unionEncoder (Dhall.encodeConstructor "A")
 
-    Dhall.embed exampleEncoder () @=? Field (Union (Dhall.Map.singleton "A" Nothing)) "A"
+    let a = Dhall.Core.makeFieldSelection "A"
+    Dhall.embed exampleEncoder () @=? Field (Union (Dhall.Map.singleton "A" Nothing)) a
+
+newtype RecursiveType a = RecursiveType (RecursiveType a)
+    deriving Generic
+
+instance FromDhall (RecursiveType a)
 
 shouldHaveWorkingRecursiveFromDhall :: TestTree
 shouldHaveWorkingRecursiveFromDhall = testGroup "recursive FromDhall instance"
     [ testCase "works for a recursive expression" $ do
         actual <- Dhall.input Dhall.auto "./tests/recursive/expr0.dhall"
 
+        expected @=? actual
+    , testCase "roundtrips (one-way)" $ do
+        let expr = Dhall.embed Dhall.inject expected
+        actual <- either throwIO pure . validationToEither . Dhall.extract Dhall.auto $ expr
+
         expected @=? actual
     , testCase "passes a shadowing sanity check" $ do
         actual <- Dhall.input Dhall.auto "./tests/recursive/expr1.dhall"
 
         expected @=? actual
+    , testCase "terminate if type is recursive (monomorphic)" $ do
+        actual <- timeout (1 * 1000000) $ do
+            !typ <- return $ Dhall.expected (Dhall.auto :: Dhall.Decoder (RecursiveType Void))
+            return typ
+        assertBool "Does not terminate!" $ isJust actual
+    , testCase "terminate if type is recursive (polymorphic)" $ do
+        actual <- timeout (1 * 1000000) $ do
+            !typ <- return $ Dhall.expected (Dhall.auto :: Dhall.Decoder (RecursiveType a))
+            return typ
+        assertBool "Does not terminate!" $ isJust actual
     ]
   where
     expected =
@@ -318,7 +353,7 @@ shouldHandleUnionsCorrectly =
       :: ( Generic a
          , Dhall.GenericToDhall (Rep a)
          , Generic b
-         , Dhall.GenericFromDhall (Rep b)
+         , Dhall.GenericFromDhall b (Rep b)
          )
       => Dhall.InterpretOptions -> Dhall.Decoder (a -> b)
     functionWithOptions options =
@@ -454,9 +489,10 @@ shouldShowCorrectErrorForInvalidDecoderInUnion =
     let assertMsg = "The exception message did not match the expected output"
 
     case inputEx of
-      Left (Dhall.ExtractErrors errs) -> case errs of
+      Left (Dhall.DhallErrors errs) -> case errs of
         (err :| []) -> case err of
           Dhall.TypeMismatch {} -> fail "The extraction using an invalid decoder failed with a type mismatch"
+          Dhall.ExpectedTypeError _ -> fail "An error occurred while determining the expected Dhall type"
           Dhall.ExtractError extractError -> assertEqual assertMsg expectedMsg extractError
         _ -> fail "The extraction using an invalid decoder failed with multiple errors"
       Right _ -> fail "The extraction using an invalid decoder succeeded"
