@@ -54,6 +54,24 @@ getTests = do
             , importDirectory </> "success/unit/asLocation/RemoteChain2A.dhall"
             , importDirectory </> "success/unit/asLocation/RemoteChain3A.dhall"
             , importDirectory </> "success/unit/asLocation/RemoteChainMissingA.dhall"
+
+              -- Skip all tests that reference httpbin.org to avoid clobbering
+              -- their servers.  These should eventually be replaced by tests
+              -- that depend on an equivalent endpoint on test.dhall-lang.org
+              -- instead of httpbin.org.
+            , importDirectory </> "failure/customHeadersUsingBoundVariable.dhall"
+            , importDirectory </> "failure/originHeadersFromRemote.dhall"
+            , importDirectory </> "failure/originHeadersFromRemoteENV.dhall"
+            , importDirectory </> "success/customHeadersA.dhall"
+            , importDirectory </> "success/noHeaderForwardingA.dhall"
+            , importDirectory </> "success/success/originHeadersA.dhall"
+            , importDirectory </> "success/originHeadersENV.dhall"
+            , importDirectory </> "success/originHeadersImportA.dhall"
+            , importDirectory </> "success/originHeadersImportENV.dhall"
+            , importDirectory </> "success/originHeadersImportFromEnvA.dhall"
+            , importDirectory </> "success/originHeadersImportFromEnvENV.dhall"
+            , importDirectory </> "success/originHeadersOverrideA.dhall"
+            , importDirectory </> "success/originHeadersOverrideENV.dhall"
             ]
 
     successTests <- Test.Util.discover (Turtle.chars <* "A.dhall") successTest (do
@@ -105,8 +123,11 @@ successTest prefix = do
 
     let expectedFailures =
             [
+              -- Importing relative to the home directory works, but I'm too
+              -- lazy to mock the home directory for testing purposes
+              importDirectory </> "success/unit/ImportRelativeToHome"
 #if !(defined(WITH_HTTP) && defined(NETWORK_TESTS))
-              importDirectory </> "success/originHeadersImportFromEnv"
+            , importDirectory </> "success/originHeadersImportFromEnv"
             , importDirectory </> "success/originHeadersImport"
             , importDirectory </> "success/originHeadersOverride"
             , importDirectory </> "success/unit/asLocation/RemoteChainEnv"
