@@ -274,13 +274,13 @@ instance Arbitrary Directory where
 
     shrink = genericShrink
 
-instance (Arbitrary s, Arbitrary a) => Arbitrary (PreferAnnotation s a) where
-    arbitrary =
-        Test.QuickCheck.oneof
-            [ pure PreferFromSource
-            , PreferFromWith <$> arbitrary
-            , pure PreferFromCompletion
-            ]
+instance Arbitrary PreferAnnotation where
+    arbitrary = Test.QuickCheck.oneof
+        [ pure PreferFromSource
+        , pure PreferFromCompletion
+        ]
+
+    shrink = genericShrink
 
 instance (Arbitrary s, Arbitrary a) => Arbitrary (RecordField s a) where
     arbitrary = lift4 RecordField
@@ -358,6 +358,8 @@ instance (Arbitrary s, Arbitrary a) => Arbitrary (Expr s a) where
             % (1 :: W "BoolEQ")
             % (1 :: W "BoolNE")
             % (1 :: W "BoolIf")
+            % (1 :: W "Bytes")
+            % (1 :: W "BytesLit")
             % (1 :: W "Natural")
             % (7 :: W "NaturalLit")
             % (1 :: W "NaturalFold")
@@ -386,10 +388,13 @@ instance (Arbitrary s, Arbitrary a) => Arbitrary (Expr s a) where
             % (1 :: W "TextShow")
             % (1 :: W "Date")
             % (1 :: W "DateLiteral")
+            % (1 :: W "DateShow")
             % (1 :: W "Time")
             % (1 :: W "TimeLiteral")
+            % (1 :: W "TimeShow")
             % (1 :: W "TimeZone")
             % (1 :: W "TimeZoneLiteral")
+            % (1 :: W "TimeZoneShow")
             % (1 :: W "List")
             % (1 :: W "ListLit")
             % (1 :: W "ListAppend")
@@ -438,8 +443,6 @@ standardizedExpression With{} =
     False
 standardizedExpression (Prefer _ PreferFromCompletion _ _) =
     False
-standardizedExpression (Prefer _ (PreferFromWith _) _ _) =
-    False
 -- The following three expressions are valid ASTs, but they can never be parsed,
 -- because the annotation will associate with `Merge`/`ListLit`/`ToMap` with
 -- higher precedence
@@ -526,7 +529,7 @@ instance Arbitrary ImportHashed where
 -- The standard does not yet specify how to encode `as Text`, so don't test it
 -- yet
 instance Arbitrary ImportMode where
-    arbitrary = Test.QuickCheck.elements [ Code, RawText, Location ]
+    arbitrary = Test.QuickCheck.elements [ Code, RawText, Location, RawBytes ]
 
     shrink = genericShrink
 
